/* width */
::-webkit-scrollbar {
    width: 0.5rem;
    /*height: 5px;*/
}

/* Track */
::-webkit-scrollbar-track {
    border-radius: 100vh;
    background: #ffffff00;
}

/* Handle */
::-webkit-scrollbar-thumb {
    background: #cbd5e0;
    border-radius: 100vh;
    border: 1px solid #edf2f7;
}

/* Handle on hover */
::-webkit-scrollbar-thumb:hover {
    background: #a0aec0;
}

body {
    overflow-x: hidden;
}

.tippy-content{
    padding: 0 !important;
}
